
# q-expansion of newform 1722.2.a.j, downloaded from the LMFDB on 16 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1722
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [-1], [0], [-1], [2], [-6], [-2], [-8], [0], [4], [4], [-2], [1], [0], [-8], [-4], [2], [8], [-6], [-16], [-14], [8], [-10], [-14], [2], [-2], [8], [12], [4], [-18], [-8], [18], [-6], [2], [-12], [0], [10], [24], [24], [0], [18], [6], [0], [14], [2], [24], [-14], [-28], [8], [-26], [6], [-12], [-2], [14], [18], [-12], [-12], [0], [-6], [-18], [14], [-14], [-10], [24], [6], [-20], [26], [2], [18], [-16], [-18], [8], [-8], [6], [36], [-24], [14], [-34], [14], [-10], [-22], [4], [24], [38], [8], [24], [-30], [38], [-16], [-8], [34], [-16], [-24], [-16], [-2], [-40], [6], [-10], [-22], [6], [2], [-8], [-16], [6], [2], [22], [-36], [-14], [-24], [46], [-48], [2], [-26], [38], [-16], [-30], [-32], [28], [12], [-42], [12], [-10], [12], [-6], [20], [-26], [0], [0], [-8], [-32], [16], [0], [-16], [16], [34], [-26], [42], [22], [12], [-38], [14], [-6], [-12], [-38], [-4], [-24], [28], [10], [-54], [-24], [-22], [-42], [-34], [40], [8], [-24], [36], [-30], [42], [-28], [52], [-42], [4], [-8], [-6], [32], [-36], [22], [2], [22], [6], [20], [0], [-50], [16], [6], [62], [-50], [8], [-34], [-24], [-18], [-52], [-10], [-32], [-16], [-26], [-58], [22], [56], [26], [-42], [-28], [-44], [52], [-10], [-22], [36], [30], [32], [2], [-40], [-50], [-14], [60], [54], [16], [-14], [-10], [-42], [-18], [-66], [-8], [18], [48], [-10], [-72], [-6], [-8], [10], [0], [16], [-6], [-16], [54], [40], [6], [48], [-16], [-8], [46], [44], [8], [38], [-64], [-48], [-2], [-68], [-32], [48], [6], [56], [-24], [34], [70], [24], [-72], [-28], [76], [-72], [58], [22], [-56], [-14], [-58], [28], [44], [64], [-46], [66], [-48], [54], [22], [38], [-38], [6], [-12], [70], [24], [48], [50], [44], [-54], [-16], [6], [24], [-38], [-28], [-30], [-48], [-80], [-16], [8], [56], [-48], [24], [46], [78], [-8], [-50], [-8], [54], [-66], [-42], [-44], [46], [-24], [-72], [-4], [-4], [78], [24], [80], [66], [72], [30], [-44], [-68], [-40], [-40], [8], [-44], [-34], [48], [48], [66], [-10], [-40], [-62], [6], [-42], [22], [76], [-4], [-74], [-18], [62], [32], [-12], [-26], [-44], [-18], [44], [8], [40], [-60], [56], [6], [-66], [24], [-16], [-54], [-50], [52], [-46], [-20], [76], [68], [-72], [0], [4], [34], [0], [32], [-56], [-46], [60], [18], [66], [0], [-10], [-10], [-52], [24], [-80], [42], [-6], [-64], [50], [-82], [44], [48], [-16], [-96], [52], [-58], [-48], [34], [-30], [34], [66], [26], [-96], [50], [-40], [-72], [12], [-30], [52], [72], [50], [-14], [-58], [-102], [24], [-2], [-56], [-54], [32], [0], [-20], [98], [8], [62], [76], [-24], [30], [18], [-32], [-66], [-70], [-70], [-42], [-66], [42], [0], [-64], [-88], [-78], [40], [-58], [98], [8], [68], [-2], [-60], [14], [62], [-28], [48]]
